; ************************************************************************
; 2 Pole Resonant Low-Pass Filter Instruments
;*************************************************************************
; 1. Table Based Rezzy Synth
; 2. Table Based PWM Rezzy Synth
; 3. Table Based Rezzy Synth with distortion
; 4. Noise Based Rezzy Synth
; 5. Buzz  Based Rezzy Synth
;*************************************************************************

sr     = 44100
kr     = 44100
ksmps  = 1
nchnls = 1

;******************************************************************
instr 1   ; Table Based Rezzy Synth

idur   = p3
iamp   = p4
ifqc   = cpspch(p5)
irez   = p7
itabl1 = p8

; Amplitude envelope
kaenv  linseg 0, .01, 1, p3-.02, 1, .01, 0

; Frequency Sweep
kfco linseg .1*p6, .5*p3, p6, .5*p3, .1*p6

; This relationship attempts to separate Freq from Res.
ka1 = 100/irez/sqrt(kfco)-1
ka2 = 1000/kfco

; Initialize Yn-1 & Yn-2 to zero
aynm1 init 0
aynm2 init 0

; Oscillator
  axn oscil iamp, ifqc, itabl1

; Replace the differential eq. with a difference eq.
  ayn = ((ka1+2*ka2)*aynm1-ka2*aynm2+axn)/(1+ka1+ka2)
  aynm2 = aynm1
  aynm1 = ayn

; Amp envelope and output
  aout = ayn * kaenv
  out aout

endin

;*************************************************************
instr 2   ; Table Based PWM Rezzy Synth

idur   = p3
iamp   = p4
ifqc   = cpspch(p5)
irez   = p7
itabl1 = p8

; Amplitude envelope
kaenv  linseg 0, .01, 1, p3-.02, 1, .01, 0

; Frequency Sweep
kfco linseg .1*p6, .5*p3, p6, .5*p3, .1*p6

; This relationship attempts to separate Freq from Res.
ka1 = 100/irez/sqrt(kfco)-1
ka2 = 1000/kfco

; Initialize Yn-1 & Yn-2 to zero
aynm1 init 0
aynm2 init 0

; PWM Oscillator
  ksine   oscil 1.5,        ifqc/440,     1
  ksquare oscil ifqc*ksine, ifqc,         2
  axn     oscil iamp,       ifqc+ksquare, itabl1

; Replace the differential eq. with a difference eq.
  ayn = ((ka1+2*ka2)*aynm1-ka2*aynm2+axn)/(1+ka1+ka2)
  aynm2 = aynm1
  aynm1 = ayn

; Amp envelope and output
  aout = ayn * kaenv
  out aout

endin

;******************************************************************
instr 3   ; Table Based Rezzy Synth with Distortion

idur   = p3
iamp   = p4
ifqc   = cpspch(p5)
irez   = p7
itabl1 = p8

; Amplitude envelope
kaenv  linseg 0, .01, 1, p3-.02, 1, .01, 0

; Frequency Sweep
kfco linseg p6, .5*p3, .2*p6, .5*p3, .1*p6

; This relationship attempts to separate Freq from Res.
ka1 = 100/irez/sqrt(kfco)-1
ka2 = 1000/kfco

; Initialize Yn-1 & Yn-2 to zero
aynm1 init 0
aynm2 init 0

; Oscillator
  axn oscil iamp, ifqc, itabl1

; Replace the differential eq. with a difference eq.
  ayn = ((ka1+2*ka2)*aynm1-ka2*aynm2+axn)/(1+ka1+ka2)

  atemp tone axn, kfco
  aclip1 = (ayn-atemp)/100000
  aclip tablei aclip1, 7, 1, .5
  aout = aclip*20000+atemp

  aynm2 = aynm1
  aynm1 = ayn

; Amp envelope and output
  out kaenv*aout

endin

;************************************************
instr 4   ; Noise Based Rezzy Synth

iamp = p4
irez = p6

; Amplitude envelope
kaenv  linseg 0, .01, 1, p3-.02, 1, .01, 0

; Frequency Sweep
kfco linseg .1*p5, .5*p3, p5, .5*p3, .1*p5

; This relationship attempts to separate Freq from Res.
ka1 = 100/irez/sqrt(kfco)-1
ka2 = 1000/kfco

; Initialize Yn-1 & Yn-2 to zero
aynm1 init 0
aynm2 init 0

; Noise Source
  axn    rand   iamp

; Replace the differential eq. with a difference eq.
  ayn = ((ka1+2*ka2)*aynm1-ka2*aynm2+axn)/(1+ka1+ka2)
  aynm2 = aynm1
  aynm1 = ayn

; Amp envelope and output
  aout = ayn * kaenv
  out aout

endin

;************************************************
instr 5   ; Buzz Based Rezzy Synth

idur   = p3
iamp   = p4
ifqc   = cpspch(p5)
irez   = p7
itabl1 = p8

; Amplitude envelope
kaenv  linseg 0, .01, 1, p3-.02, 1, .01, 0

; Frequency Sweep
kfco linseg .1*p6, .5*p3, p6, .5*p3, .1*p6

; This relationship attempts to separate Freq from Res.
ka1 = 100/irez/sqrt(kfco)-1
ka2 = 1000/kfco

; Initialize Yn-1 & Yn-2 to zero
aynm1 init 0
aynm2 init 0

; Buzz Source
;               pitch,  amp, #partials, function
  axn    buzz    iamp, ifqc,  itabl1,      1

; Replace the differential eq. with a difference eq.
  ayn = ((ka1+2*ka2)*aynm1-ka2*aynm2+axn)/(1+ka1+ka2)
  aynm2 = aynm1
  aynm1 = ayn

; Amp envelope and output
  aout = ayn * kaenv
  out aout

endin
