#-------------------------------------------------------------------------
# Automatic Csound Score Generation by Hans Mikelson January 1999
#
# This program generates a fractal csound score for use in granular synthesis.
#-------------------------------------------------------------------------
# SUBROUTINES
#-------------------------------------------------------------------------

sub circsq
 {
  my $sta     = $_[0];
  my $dur     = $_[1];
  my $fqc     = $_[2];
  my $xcorner = $_[3];
  my $ycorner = $_[4];
  my $step    = $_[5];
  $nsteps     = 10;
  $numfqc     = 20;
  $amp        = 2000;

  $sta1       = $sta;
  $x=$xcorner;
  for ( $i=0; $i<$nsteps; $i++)
   {
    $y=$ycorner;
    for ( $j=0; $j<$nsteps; $j++)
     {
      $z    = ($x*$x+$y*$y)%$numfqc;
      $fqc1 = $z*$fqc/10+$fqc;
      $pan  = rand(1);
      print SCO "i1 $sta $dur $amp $fqc1 $pan\n";
      $y+=$step;
      $sta=rand($dur)+.02;
     }
    $x+=$step;
   }
 }

# Feather Fractal
sub ffract
 {
  $nsteps = 100;
  $h   = .001;
  $a   = -.48;
  $c   = 2-2*$a;
  $b   = .93;
  $x   = 3;
  $y   = 0;
  $w   = $a*3+$c*.9;
  $amp = 5000;

  for ( $i=0; $i<$nsteps; $i++)
   {
    $z   = $x;
    $x   = $b*$y+$w;
    $u   = $x*$x;
    $w   = $a*$x+$c*$u/(1+$u);
    $y   = $w-$z;
    $sta = (10+$z)/2;
    $dur = (10+$y)/4;
    $fqc = 440+$x*12;
    $pan = rand(1);
    $fco = 2000+$w*10;
    $rez = (10+$x);
    print SCO "i3 $sta $dur $amp $fqc $pan $fco $rez\n";
   }
 }

# Henon Attractor
sub henon
 {
  $a = $_[0];
  $b = $_[1];
  $xx = 1 + $y - $a*$x*$x;
  $yy = $b*$x;
  $x = $xx;
  $y = $yy;
 }

sub hscore
 {
  $stao = $_[0];
  $stam = $_[1];
  $duro = $_[2];
  $durm = $_[3];
  $ampo = $_[4];
  $ampm = $_[5];
  $fqco = $_[6];
  $fqcm = $_[7];
  $iter = $_[8];

  $x = 0;
  $y = 0;
  $max = .1;

  for ($i=0; $i<$iter; $i++)
   {
    henon(1.4, .3);
    $ax = abs($x);
    $ay = abs($y);
    if ($ax > $max) {$max = $ax;}
    if ($ay > $max) {$max = $ay;}
    $sta = $stam*int($sf*$ax)/$sf + $stao;
    $dur = $durm*int($sf*$ay)/$sf + $duro;
    $amp = $ampm*int($sf*$ax)/$sf + $ampo;
    $fqc = $fqcm*int($sf*$ay)/$sf + $fqco;
    $pan = ($ay - $ax)/$max/2+.5;

    print SCORE "i1 $sta $dur $amp $fqc $pan\n";
   }
}

# Main Program begins here

$scorefile = "fscore.sco";
open (SCORE, ">$scorefile");
print SCORE "f1 0 8192 10 1 \n";

$sf = 1000;
#      Sta    Dur     Amp         Fqc         Iterates
hscore(0,  2, .1, .2, 100,  8000, 600,  100,  160);
hscore(2,  1, .1, .4, 200,  4000, 400,  400,  60);
#hscore(5,  2, .3, 2, 0,    1000, 480,  1000, 30);
#hscore(6,  1, .1, 7, 0,    1000, 200,  200,  200);
#hscore(7,  2, .5, 4, 0,    2000, 200,  1000, 120);
#hscore(8,  3, .1, 4, 0,    3000, 300,  400,  20);
#hscore(9,  2, .3, 2, 0,    2000, 280,  300,  40);
#hscore(10, 2, .2, 3, 0,    3000, 100,  200,  80);

close SCORE;

# perl fscore.pl
# cs fscore.orc fscore.sco
# soundeditor test
